@@ -16,8 +16,10 @@ import (
 // as the exit status.
 //
 func (eng *Engine) ServeHTTP(w http.ResponseWriter, r *http.Request) {
-	jobName := path.Base(r.URL.Path)
-	jobArgs, exists := r.URL.Query()["a"]
+	var (
+		jobName         = path.Base(r.URL.Path)
+		jobArgs, exists = r.URL.Query()["a"]
+	)
 	if !exists {
 		jobArgs = []string{}
 	}
